__title__ = "jvig"
__description__ = "GTFS Viewer, created using Flask."
__version__ = "1.1.1"
__url__ = "https://github.com/MKuranowski/jvig"
__author__ = "Mikołaj Kuranowski"
__email__ = "mkuranowski+pypackages@gmail.com"
__copyright__ = "© Copyright 2020-2024 Mikołaj Kuranowski"
__license__ = "GNU General Public License v3.0 or later"
